Attribute VB_Name = "modDSP"
'=======================================================================================
' Descrizione.....: Collezione di routines e costanti di utilita'
'                   per il Digital Signal Processing; non tutte le
'                   routines e costanti di questo modulo vengono,
'                   necessariamente, usate.
'                   Versione per la decodifica di sequenze tonali.
' Nome dei Files..: modDSP, modUtilita.bas
' Data............: 13/06/1999
' Revisione.......: 26/08/2003
' Revisione.......: 08/03/2004 (sequenze tonali)
' Versione........: 1.0 a 32 bits.
' Sistema.........: VB6 (SP5) sotto Windows XP (SP2).
' Scritto da......: F. Languasco 
' E-Mail..........: MC7061@mclink.it
' DownLoads a.....: http://www.flanguasco.org
'=======================================================================================
'
'   Le routines di questo modulo sono un sotto insieme
'   di quelle contenute nel progetto FiltCalc (sito dei
'   DownLoads) e sono state modificate per adattarle a
'   questa applicazione.
'
Option Explicit
'
' ---- Trasformata DFT_1 ------------------------------------------------------
'
Private NcDFT_1&    ' N di campioni - 1 per il calcolo della DFT.
Private NFreqDFT_1& ' N di frequenze - 1 a cui calcolare la DFT.
Private NormDFT#    ' Fattore di normalizzazione sul N di campioni.
Private SinMat#()   ' Matrice dei seni pre-calcolati per DFT_1.
Private CosMat#()   ' Matrice dei coseni pre-calcolati per DFT_1.
'
' ---- Spettro mobile (sliding spectrum) --------------------------------------
'   Implementazione dei circuiti tratti da:
'    "Theory and Application of Digital Signal Processing"
'    di L. Rabiner e B. Gold. - pg. 382, 383.
'
Private NcSpm&                  ' N di campioni per il calcolo dello spettro mobile.
Private NormSpm#                ' Fattore di normalizzazione sul N di campioni.
'
                                ' Coefficienti per il calcolo con convoluzione diretta:
Private z1() As Complex         '  vettore dei coefficienti di calcolo dello spettro mobile.
Private SmRE#()                 '  registro a scorrimento dei campioni del segnale.
'
                                ' Coefficienti per il calcolo  ricorsivo:
Private Sn_1 As Complex         '  Sn * Z^-1.
Private z1_1 As Complex         '  z1(0)^-1
Private z1_N As Complex         '  z1(0)^-N
Private Ic&, Kc&, NcSpm1&       '  indici in SmRE() per buffer circolare.
Private Xz1_N As Complex        '  variabili di appoggio.
Private Yz1_1 As Complex        '      "     "     "
Private SnI As Complex          '      "     "     "
Private Sn As Complex           '      "     "     "
'
' Per multi frequenza:
Private NFreqSPM_1&             ' N di frequenze - 1 a cui calcolare gli spettri.
                                ' Coefficienti per il calcolo  ricorsivo multi frequenza:
Private Sn_MF_1() As Complex    '  vettore degli (NFreqSPM_1 + 1) Sn * Z^-1
Private z1_MF_1() As Complex    '  z1()^-1
Private z1_MF_N() As Complex    '  z1()^-N
'
Private SqRE#()                 ' Registro a scorrimento dei campioni del segnale al quadrato.
Private VMed#                   ' Valore medio corrente.
'
' ---- Filtri FIR -------------------------------------------------------------
'   Algoritmi tradotti ed adattati dal FORTRAN da:
'     "Digital Filters and their Applications"
'     di V. Cappellini, A. G. Constantinides, P. Emiliani.
'     Window method:  pg. 350.
'
Private Hc!()        ' Vettore dei coefficienti del Filtro.
'
Private WF#()        ' Tabella dei coefficienti per Windowing.
'
Private SRE!()       ' Registro a scorrimento dei Dati da Filtrare.
'
Private Type Window_Type    '
    Nome As String          ' Nome della "Window":
    PMin As Double          ' Valore Min. del parametro associato.
    PMax As Double          ' Valore Max. del parametro associato.
    PCor As Double          ' Valore corrente del parametro associato.
End Type
'
Private Const A0# = 0.99938     ' Coefficienti per la
Private Const A1# = 0.041186    ' Weber Window.
Private Const A2# = -1.637363   '
Private Const A3# = 0.828217    '
Private Const B0# = 1.496611    '
Private Const B1# = -1.701521   '
Private Const B2# = 0.372793    '
Private Const B3# = 0.0650621   '
'
Private Const FiltroErrFIR$ = "Le Routines di calcolo hanno trovato" & vbNewLine _
    & "una condizione imprevista." & vbNewLine _
    & "Provare a rivedere i parametri del Filtro." & vbNewLine & vbNewLine
'
' ---- Filtri IIR -------------------------------------------------------------
'   Gli algoritmi per la sintesi di filtri del tipo Butterworth
'   e Chebyshev sono stati tradotti ed adattati dal FORTRAN da:
'     "Digital Filters and their Applications"
'     di V. Cappellini, A. G. Constantinides, P. Emiliani.
'     pg. 367, 368, 369.
'   Metodi di calcolo dei filtri elementari.
'   - Algoritmi di trasformazione da "Digital Signal Processing"
'     di W. D. Stanley - pg. 172, 173, 174.
'   - La funzione di trasferimento del risuonatore reale parallelo
'     e' tratta da "Teoria delle Reti Elettriche", Appunti dai
'     corsi del Politecnico di Torino - pg. (1.3)1 e seg.
'   - L' idea della sostituzione degli zeri per il "Notch Filter"
'     proviene da: http://www-users.cs.york.ac.uk/~fisher/mkfilter/res.html
'
Private NK&, NCel&               ' Ordine e numero di sezioni del filtro.
Private Ac!()                    ' Coefficienti del Filtro.
Private Bc!()                    '      "        "     "
'
Private Const NCMax& = 20        ' N. Massimo di sezioni del Filtro.                                                                         
'
Private w!()                     ' Registri delle sezioni del Filtro.
'
Private CEB#(1 To 2 * NCMax + 1) ' Vettori in uso durante la sintesi.
Private AN#(0 To 4, 0 To 2)      '    "        "     "          "
Private FINA#(0 To 2)            '    "        "     "          "
Private FINB#(0 To 2)            '    "        "     "          "
'
Private Const FiltroErrIIR$ = "Le Routines di calcolo hanno trovato" & vbNewLine _
    & "una condizione imprevista." & vbNewLine _
    & "Provare a rivedere i parametri del Filtro." & vbNewLine & vbNewLine
Public Sub LeggiCoefficienti_IIR(ByRef NK_U As Long, ByRef NCel_U As Long, _
    ByRef W_U() As Single, ByRef Ac_U() As Single, ByRef Bc_U() As Single)
'
'   Con questa routine e' possibile ritornare, per uso locale, i
'   coefficienti di un filtro IIR calcolati precedentemente dalla
'   Function SintesiIIR_BT():
'
'   Routine usata in AudioCardDSP (27/04/2009)
'
    NK_U = NK       ' Ordine
    NCel_U = NCel   ' e numero di sezioni del filtro.
    W_U() = w()     ' Registri delle sezioni del filtro.
    Ac_U() = Ac()   ' Coefficienti del filtro.
    Bc_U() = Bc()   '      "        "     "
'
'
'
End Sub
Public Function SintesiIIR_BT(ByVal Trans$, ByVal Proty$, _
    ByRef FCt As Double, ByVal FAt As Double, ByVal Att As Double, ByVal Rip As Double, _
    ByVal F1 As Double, ByVal F2 As Double, Optional ByVal Q As Double, _
    Optional ByRef AttTot As Double, Optional ByRef ALFA As Double) As Boolean
'
'   Sintesi del Filtro richiesto con il metodo
'   di Trasformazione Bilineare.
'
'   Routine modificata per selezione toni.
'   --------------------------------------
'
'   Parametri in ingresso.
'    Trans$:    Tipo di Trasformazione richiesta:
'                "No Transformation"               JBT = 0
'                "Low Pass -> Low Pass"            JBT = 1
'                "Low Pass -> High Pass"           JBT = 2
'                "Low Pass -> Band Pass"           JBT = 3
'                "Low Pass -> Band Stop"           JBT = 4
'               Filtri elementari:
'                "RC Type Low Pass Filter"         JBT = 5
'                "RC Type High Pass Filter"        JBT = 6
'                "Resonator"                       JBT = 7
'                "Notch Filter"                    JBT = 8
'                "U(t) = aI(t) + (1-a)U(t-1)"    JBT = 9
'    Proty$:    Tipo di prototipo Passa Basso:
'                "Butterworth"
'                "Chebyshev"
'    FCt:       Frequenza di taglio (con atten. di -3 [dB])
'               o di risonanza (solo per JBT = 7 e JBT = 8).
'    FAt:       Per JBT < 5 e' la frequenza alla quale
'               l' attenuazione deve essere di almeno Att# [dB].
'    Att:       Attenuazione desiderata in [dB] alla frequenza FAt#.
'    Rip:       Ondulazione in Banda Passante espressa come
'               percentuale del guadagno (solo per Chebyshev).
'    F1:        Prima frequenza di taglio dopo la trasformazione.
'    F2:        Seconda frequenza di taglio dopo la trasformazione.
'    Q:         Solo per JBT = 7 e JBT = 8, e' il Q del risonatore.
'
'   Parametri in uscita
'    AttTot:    Attenuazione ottenuta in [dB] alla frequenza FAt#.
'    FCt:       Frequenza di taglio ottenuta per filtro JBT = 9 con ALFA dato.
'    ALFA:      Alfa ottenuto per filtro JBT = 9 con FCt dato.
'
'   I coefficienti del filtro vengono calcolati nelle
'   matrici Ac(0 To NK, 1 To NCel) e Bc(0 To NK, 1 To NCel)
'   con NK = ordine del filtro e NCel = numero di sezioni.
'
    Dim N&, I&, K&, NC&, M$
    Dim Om#, Cm#, a#, A0#, A1#, A2#, B0#, B1#, B2#      ' Variabili per filtri elementari.
    Dim F#, FF#, FCTA#, FATA#, Eps#, ATL#, res#         ' Variabili per
    Dim ab#, AT#, Bt#, TETAP#, P1#, P2#, P3#            ' filtri di tipo
    Dim CAPPA#, COSTA#, TOT#, CC#, AL#, Rn#             ' Butterworth e Chebyshev.
'
    On Error GoTo SintesiIIR_BT_ERR
'
'-- Filtri elementari ------------------------------------------------------------------
    If Trans$ = "RC Type Low Pass Filter" _
    Or Trans$ = "RC Type High Pass Filter" _
    Or Trans$ = "U(t) = aI(t) + (1-a)U(t-1)" Then
        ' Calcolo di tipi di Filtro elementari del 1 ordine.
        ' Vedi Sub Note:
        Om = PI2 * FCt  ' Pulsazione di taglio [rad/s].
'
        Select Case Trans$
            Case "RC Type Low Pass Filter"
            Cm = Om / Tan(PI * FCt) ' Costante di mappatura.
            a = 1# + Cm / Om
            A0 = 1# / a
            A1 = 1# / a
            B0 = 1#
            B1 = (1# - Cm / Om) / a
'
            Case "RC Type High Pass Filter"
            Cm = Om / Tan(PI * FCt) ' Costante di mappatura.
            a = 1# + Cm / Om
            A0 = (Cm / Om) / a
            A1 = -(Cm / Om) / a
            B0 = 1#
            B1 = (1# - Cm / Om) / a
'
            Case "U(t) = aI(t) + (1-a)U(t-1)"
            ' a := ALFA
            If ALFA < 0# Then
                ' E' data FCt; calcola ALFA per avere la FCt voluta:
                ALFA = Cos(Om) - 1# + Sqr(Cos(Om) ^ 2 - 4# * Cos(Om) + 3#)
            ElseIf ALFA < 2# * (Sqr2 - 1#) Then
                ' E' data ALFA; calcola la FCt (se < 0.5) corrispondente:
                Om = Acos(((ALFA ^ 2) + 2# * ALFA - 2#) / (2# * ALFA - 2#))
                FCt = Om / PI2
            End If
            A0 = ALFA
            A1 = 0#
            B0 = 1#
            B1 = -(1# - ALFA)
        End Select
'
        NCel = 1
        NK = 1
        ReDim Ac(0 To NK, 1 To NCel)   ' Coefficienti del Filtro
        ReDim Bc(0 To NK, 1 To NCel)   ' calcolati in Doppia Precisione.
        Ac(0, 1) = A0
        Ac(1, 1) = A1
        Bc(0, 1) = B0
        Bc(1, 1) = B1
        AttTot = 1#
'
        GoTo SintesiIIR_BT_End
'
    ElseIf Trans$ = "Resonator" _
        Or Trans$ = "Notch Filter" Then
        ' Calcolo di tipi di Filtro elementari del 2 ordine.
        ' Vedi Sub Note:
        Om = PI2 * FCt          ' Pulsazione di risonanza [rad/s].
        Cm = Om / Tan(PI * FCt) ' Costante di mappatura.
'
        ' Calcola i poli:
        a = 1# + Cm / (Q * Om) + (Cm ^ 2 / Om ^ 2)
        B0 = 1#
        B1 = 2# * (1# - Cm ^ 2 / Om ^ 2) / a
        B2 = (1# - Cm / (Q * Om) + (Cm ^ 2 / Om ^ 2)) / a
'
        Select Case Trans$
            Case "Resonator"
            ' Mette due zeri reali a  1:
            A0 = Cm / a
            A1 = 0#
            A2 = -Cm / a
'
            Case "Notch Filter"
            ' Mette una coppia di zeri complessi coniugati:
            Rn = Sqr(B2)
                            ' TETAP = Acos(-B1 / (2# * Rn))
            A0 = 1#
            A1 = B1 / Rn    ' -2 * Cos(TETAP)
            A2 = 1#         ' Cos(TETAP) ^ 2 + Sin(TETAP) ^ 2
        End Select
'
        NCel = 1
        NK = 2
        ReDim Ac(0 To NK, 1 To NCel)   ' Coefficienti del Filtro
        ReDim Bc(0 To NK, 1 To NCel)   ' calcolati in Doppia Precisione.
        Ac(0, 1) = A0
        Ac(1, 1) = A1
        Ac(2, 1) = A2
        Bc(0, 1) = B0
        Bc(1, 1) = B1
        Bc(2, 1) = B2
        AttTot = 1#
'
        GoTo SintesiIIR_BT_End
    End If
'
'-- Filtri Butterworth e Chebyshev -----------------------------------------------------
    FCTA = Tan(PI * FCt)
    FATA = Tan(PI * FAt)
    FF = FATA / FCTA
    If Proty$ = "Chebyshev" Then Eps = Sqr(1# / (1# - Rip / 100#) ^ 2# - 1#)
    ATL = 10# ^ (Att / 10#)
'
'       COMPUTATION OF THE ORDER OF THE FILTER.
'       ---------------------------------------
'
    N = 0
    Do
        N = N + 1
        Select Case Proty$
            Case "Butterworth"
            res# = 1# / (1# + FF ^ (2 * N))
'
            Case "Chebyshev"
            CEB(1) = 1#
            CEB(2) = FF
            If N > 1 Then
                   F = FF + FF
                   For I = 2 To N
                       CEB(I + 1) = F * CEB(I) - CEB#(I - 1)
                   Next I
            End If
            res = 1# / (1# + (Eps * CEB(N + 1)) ^ 2)
        End Select
    Loop While res >= ATL
'
    If N Mod 2 = 1 Then
        N = N + 1
        Select Case Proty$
            Case "Butterworth"
            res = 1# / (1# + FF ^ (2 * N))
'
            Case "Chebyshev"
            CEB(1) = 1
            CEB(2) = FF
            F = FF + FF
            For I = 2 To N
                CEB(I + 1) = F * CEB(I) - CEB(I - 1)
            Next I
            res = 1# / (1# + (Eps * CEB(N + 1)) ^ 2)
        End Select
    End If
'
    If N > 2 * NCMax Then
        M$ = "**** ERRORE ****" & vbNewLine
        M$ = M$ & vbNewLine & "Per ottenere il Filtro richiesto" & vbNewLine
        M$ = M$ & "si devono usare " & Str$(N / 2) & " Sezioni." & vbNewLine
        M$ = M$ & "Il numero Massimo e' " & Str$(NCMax) & "." & vbNewLine
        M$ = M$ & vbNewLine & "Provare a variare i dati del Filtro."
        MsgBox M$, vbCritical, " Sintesi IIR"
        SintesiIIR_BT = False
        DoEvents
        Exit Function
    End If
'
    AttTot = 10# * Log(res) / Log10
    NCel = N / 2
    NK = Switch(Trans$ = "No Transformation", 2, _
                Trans$ = "Low Pass -> Low Pass", 2, _
                Trans$ = "Low Pass -> High Pass", 2, _
                Trans$ = "Low Pass -> Band Pass", 4, _
                Trans$ = "Low Pass -> Band Stop", 4)
    ReDim Ac(0 To NK, 1 To NCel)    ' Coefficienti del Filtro.
    ReDim Bc(0 To NK, 1 To NCel)    ' Calcolati in Doppia Precisione.
    ReDim POLRE(1 To NCel)          ' Vettori in uso durante la Sintesi.
    ReDim POLIM(1 To NCel)          '    "        "     "          "
    ReDim w(0 To NK, 1 To NCel)
'
'       POLE POSITION EVALUATION.
'       -------------------------
'
    AT = FCTA
    Bt = FCTA
    If Proty$ = "Chebyshev" Then
        ab = ((1# + Sqr(1# + Eps ^ 2)) / Eps) ^ (1# / N)
        AT = (ab - 1# / ab) * AT / 2#
        Bt = (ab + 1# / ab) * Bt / 2#
    End If
    For NC = 1 To NCel
        TETAP = PI# * (2# * NC - 1#) / (2# * N)
        POLRE(NC) = -AT * Cos(TETAP)
        POLIM(NC) = Bt * Sin(TETAP)
    Next NC
    For NC = 1 To NCel
        P1 = POLIM(NC) ^ 2
        P2 = (1# - POLRE(NC)) ^ 2
        P3 = POLRE(NC) ^ 2
        POLRE(NC) = (1# - P3 - P1) / (P2 + P1)
        POLIM(NC) = (2# * POLIM(NC)) / (P2 + P1)
    Next NC
'
'       EVALUATION OF THE SECOND ORDER SECTION COEFFICIENTS.
'       ----------------------------------------------------
'
    For NC = 1 To NCel
        Bc(0, NC) = 1#
        Bc(1, NC) = -2# * POLRE(NC)
        Bc(2, NC) = POLRE(NC) * POLRE(NC) + POLIM(NC) * POLIM(NC)
    Next NC
'
    Select Case Proty$
        Case "Butterworth"
        COSTA = 1#
'
        Case "Chebyshev"
        COSTA = (1# - Rip / 100#) ^ (1# / NCel)
    End Select
'
    For NC = 1 To NCel
        TOT = 4# / (Bc(0, NC) + Bc(1, NC) + Bc(2, NC))
        CC = TOT / COSTA
        Ac(0, NC) = 1# / CC
        Ac(1, NC) = 2# / CC
        Ac(2, NC) = 1# / CC
    Next NC
'
'       FREQUENCY TRANSFORMATION.
'       -------------------------
'
    Select Case Trans$
'
        Case "No Transformation"
        GoTo SintesiIIR_BT_End
'
        Case "Low Pass -> Low Pass"
        AL = Sin(PI * (FCt - F1)) / Sin(PI * (FCt + F1))
        A0 = -AL
        A1 = 1#
        A2 = 0#
        B0 = A1
        B1 = A0
        B2 = 0#
'
        Case "Low Pass -> High Pass"
        AL = -Cos(PI * (FCt + F1)) / Cos(PI * (FCt - F1))
        A0 = -AL
        A1 = -1#
        A2 = 0#
        B0 = -A1
        B1 = -A0
        B2 = 0#
'
        Case "Low Pass -> Band Pass"
        AL = Cos(PI * (F2 + F1)) / Cos(PI * (F2 - F1))
        CAPPA = Tan(PI * FCt) / Tan(PI * (F2 - F1))
        A0 = -(CAPPA - 1#) / (CAPPA + 1#)
        A1 = 2# * AL * CAPPA# / (CAPPA + 1#)
        A2 = -1#
        B0 = -A2
        B1 = -A1
        B2 = -A0
'
        Case "Low Pass -> Band Stop"
        AL = Cos(PI * (F2 + F1)) / Cos(PI * (F2 - F1))
        CAPPA = Tan(PI * FCt) * Tan(PI * (F2 - F1))
        A0 = (1# - CAPPA) / (1# + CAPPA)
        A1 = -2# * AL / (1# + CAPPA)
        A2 = 1#
        B0 = A2
        B1 = A1
        B2 = A0
    End Select
'
    AN(0, 0) = B0 * B0
    AN(1, 0) = 2# * B0 * B1
    AN(2, 0) = B1 * B1 + 2# * B0 * B2
    AN(3, 0) = 2# * B1 * B2
    AN(4, 0) = B2 * B2
    AN(0, 1) = A0 * B0
    AN(1, 1) = A0 * B1 + A1 * B0
    AN(2, 1) = A0 * B2 + A1 * B1 + A2 * B0
    AN(3, 1) = A1 * B2 + A2 * B1
    AN(4, 1) = A2 * B2
    AN(0, 2) = A0 * A0
    AN(1, 2) = 2# * A0 * A1
    AN(2, 2) = A1 * A1 + 2# * A0 * A2
    AN(3, 2) = 2# * A1 * A2
    AN(4, 2) = A2 * A2
'
    For NC = 1 To NCel
        For K = 0 To 2
            FINA(K) = Ac(K, NC)
            FINB(K) = Bc(K, NC)
        Next K
        For K = 0 To NK
            Ac(K, NC) = 0#
            Bc(K, NC) = 0#
            For I = 0 To 2
                Ac(K, NC) = AN(K, I) * FINA(I) + Ac(K, NC)
                Bc(K, NC) = AN(K, I) * FINB(I) + Bc(K, NC)
            Next I
        Next K
    Next NC
    For NC = 1 To NCel
        Rn = Bc(0, NC)
        For K = 0 To NK
            Ac(K, NC) = Ac(K, NC) / Rn
            Bc(K, NC) = Bc(K, NC) / Rn
        Next K
    Next NC
'
'
SintesiIIR_BT_End:
SintesiIIR_BT_ERR:
    SintesiIIR_BT = (Err.Number = 0)
    If (Err.Number <> 0) Then
        M$ = FiltroErrIIR$ & vbNewLine
        M$ = M$ & "Errore " & Str$(Err.Number) & vbNewLine
        M$ = M$ & Err.Description
        MsgBox M$, vbCritical, " modDSP: SintesiIIR_BT"
    End If
'
'
'
End Function
Private Sub zNote()
'
'   Metodi di calcolo dei filtri elementari.
'   - Algoritmi di trasformazione da "Digital Signal Processing"
'     di W. D. Stanley - pg. 172, 173, 174.
'   - La funzione di trasferimento del risuonatore reale parallelo
'     e' tratta da "Teoria delle Reti Elettriche", Appunti dai
'     corsi del Politecnico di Torino - pg. (1.3)1 e seg.
'   - L' idea della sostituzione degli zeri per il "Notch Filter"
'     proviene da: http://www-users.cs.york.ac.uk/~fisher/mkfilter/res.html
'
'---------------------------------------------------------------------------------------
'   "RC Type Low Pass Filter":
'   La funzione di trasferimento e':
'
'   U(p)       1                    con:
'   ---- = -----------               Fc = 1/(2*PI*RC)   frequenza di taglio a -3 [dB] [Hz].
'   I(p)   1 + RC*p                  C, R               capacita' e resistenza
'                                                       del circuito.
'
'   Le formule di trasformazione (1) da:
'
'           A0 + A1*p                 1
'    G(p) = ---------   -->  -------------------
'           B0 + B1*p        1 + (1/(2*PI*Fc))*p
'   a:
'                    -1
'           a0 + a1*z
'    H(z) = -----------
'                    -1
'           1  + b1*z
'
'   sono:                           con:
'    Cm = 2*PI*FCt/Tan(2*PI*FCt/2)   FCt = Fc/Fs  -->  0 <= FCt <= 0.5
'    A  =  B0 + B1*Cm                A0 = 1
'    a0 = (A0 + A1*Cm)/A             A1 = 0
'    a1 = (A0 - A1*Cm)/A             B0 = 1
'    b1 = (B0 - B1*Cm)/A             B1 = 1/(2*PI*FCt)
'
'
'---------------------------------------------------------------------------------------
'   "RC Type High Pass Filter":
'   La funzione di trasferimento e':
'
'   U(p)      RCp                   con:
'   ---- = -----------               Fc = 1/(2*PI*RC)   frequenza di taglio a -3 [dB] [Hz].
'   I(p)   1 + RC*p                  C, R               capacita' e resistenza
'                                                       del circuito.
'
'   si usano le formule di trasformazione (1) con:
'                                              FCt = Fc/Fs  -->  0 <= FCt <= 0.5
'                                              A0 = 0
'                                              A1 = 1/(2*PI*FCt)
'                                              B0 = 1
'                                              B1 = 1/(2*PI*FCt)
'
'
'---------------------------------------------------------------------------------------
'   "Resonator":
'   La funzione di trasferimento e':
'
'    U(p)                  P                    con:
'    ---- = ---------------------------------    Oc = 1/Sqr(L*C)    pulsazione di risonanza [rad/s].
'    I(P)   1 + (1/(Q*Oc))*P + (1/(Oc^2))*P^2    Fc = Oc/(2*PI)     frequenza di risonanza [Hz].
'                                                L, C, Rp           induttanza, capacita' e
'                                                                   resistenza in parallelo
'                                                                   del circuito risonante.
'                                                Q = Oc*Rp*C        coefficiente di risonanza.
'
'   Le formule di trasformazione (2) da:
'
'           A0 + A1*p + A2*p^2                         p
'    G(p) = ------------------   -->  ---------------------------------
'           B0 + B1*p + B2*p^2        1 + (1/(Q*Oc))*P + (1/(Oc^2))*P^2
'   a:
'                    -1     -2
'           a0 + a1*z + a2*z
'    H(z) = ------------------
'                    -1     -2
'           1  + b1*z + b2*z
'
'   sono:                             con:
'    Cm = 2*PI*FCt/Tan(2*PI*FCt/2)     FCt = Fc/Fs  -->  0 <= FCt <= 0.5
'    A  =  B0 + B1*Cm + B2*Cm^2        A0 = 0
'    a0 = (A0 + A1*Cm + A2*Cm^2)/A     A1 = 1
'    a1 = (2*A0 - 2*A2*Cm^2)/A         A2 = 0
'    a2 = (A0 - A1*Cm + A2*Cm^2)/A     B0 = 1
'    b1 = (2*B0 - 2*B2*Cm^2)/A         B1 = 1/(2*PI*FCt*Q)
'    b2 = (B0 - B1*Cm + B2*Cm^2)/A     B2 = 1/(2*PI*FCt)^2
'
'---------------------------------------------------------------------------------------
'   "Notch Filter":
'   L' algoritmo precedente costruisce una funzione di trasferimento H(z) avente
'   una coppia di zeri reali a  1; questo consente di avere guadagno zero alle
'   basse ed alle alte frequenze.
'   Il "Notch Filter" viene ricavato dal "Resonator" sostituendo questa coppia di
'   zeri reali con due zeri complessi coniugati, giacenti sul cerchio unitario e
'   con argomento uguale a quello dei poli (anch' essi complessi coniugati).
'
'   Se Zp = r*Exp( i*Theta) sono i poli di H(z), gli zeri devono essere:
'
'    Zz = Exp( i*Theta)
'
'   Viene usata la relazione:
'
'                          -1                        -1
'    (1 - r*Exp(+i*Theta)*z  )*(1 - r*Exp(-i*Theta)*z  ) =
'
'                          -1   2   -2           -1      -2
'    = 1 - 2*r*Cos(Theta)*z  + r * z   = 1 + b1*z  + b2*z       (i = Sqr(-1))
'
'   da cui:
'
'    r = Sqr(b2)
'    Theta = Acos(-b1/(2*Sqr(b2))
'
'   e quindi:
'
'             -1     -2                       -1  -2
'    a0 + a1*z + a2*z  = 1 + (b1/(2*Sqr(b2))*z + z
'
'---------------------------------------------------------------------------------------
'   "U(t) = aI(t) + (1-a)U(t-1)"
'   La funzione di trasferimento e':
'
'                a
'    H(z) = ----------------
'                         -1
'           1  - (1 - a)*z
'
'   Usando, invertite, le formule di trasformazione (1) si ha:
'
'              a*Cm - a*p
'    G(p) = ---------------             con: Cm = Oc/Tan(Oc/2)
'           a*Cm + (2 - a)*p                 Oc = 2*PI*Fc
'
'   La pulsazione di taglio Oc si ha per:
'
'          |a*Cm - a*i*Oc|       1
'    ----------------------- = ------   con: p = i*Oc
'      |a*Cm + (2 - a)*i*Oc|   Sqr(2)        i = Sqr(-1)
'
'   da cui si ricava il valore di a necessario per
'   ottenere la frequenza di taglio FCt voluta:
'
'    a = Cos(Oc) - 1 + Sqr(Cos(Oc)^2 - 4*Cos(Oc) + 3)   sempre con: Oc  = 2*PI*FCt
'                                                                   FCt = Fc/Fs
'
'   Se invece viene dato a, si ottiene una FCt corrispondente:
'
'    FCt = Acos(((a^2) + 2*a - 2)/(2*a - 2))/(2*PI)     solo per:   a   < 2*(Sqr(2) - 1)
'                                                       e:          FCt < 0.5
'
'
'
End Sub
Public Function IIR(ByVal Vi As Single) As Single
'
'   Filtrazione IIR del Segnale:
'   Devono essere state definite, precedentemente, le matrici
'   Ac(0 To NK, 1 To NCel) e Bc(0 To NK, 1 To NCel) contenenti
'   i coefficienti del filtro ed una matrice W(0 To NK, 1 To NCel)
'   da usare come registro a scorrimento dei campioni da filtrare.
'
'   Routine modificata per selezione toni.
'   --------------------------------------
'
    Dim C&, K&, Y!
'
    Y = Vi
    For C = 1 To NCel
        w(0, C) = Y
        Y = 0!
        For K = NK To 1 Step -1
            w(0, C) = w(0, C) - Bc(K, C) * w(K, C)
            Y = Y + Ac(K, C) * w(K, C)
            w(K, C) = w(K - 1, C)
        Next K
        Y = Y + Ac(0, C) * w(0, C)
    Next C
'
    IIR = Y
'
'
'
End Function
Public Sub DFT_1(D() As Double, ByRef r() As Double, ByRef X() As Double, _
    ByRef S() As Double, ByRef F() As Double, ByRef SMax As Double, _
    ByRef ISMax As Long, ByRef SMed As Double)
'
'   Definizioni:
'
'   DFT = Discrete Fourier Transform.
'
'   Entra con:
'       NVAL   = Numero di dati in D() (passato in DFT_1_Init).
'       D()    = Vettore dei dati Reali del Segnale(t).
'       NFRE   = Numero dei valori di Frequenza da calcolare (passato in DFT_1_Init).
'
'   Esce  con:
'       R()    = Vettore dei valori Reali      della Trasformata(f).
'       X()    = Vettore dei valori Immaginari della Trasformata(f).
'       S()    = Vettore dello Spettro di Amp. della Trasformata(f).
'       F()    = Vettore delle Fasi            della Trasformata(f) [rad].
'       SMax  = Valor massimo dello Spettro di Ampiezza.
'       ISMax = Indice del valor massimo dello Spettro di Ampiezza.
'       SMed  = Valor medio di S() con l' esclusione del valore
'               corrispondente a ISMax.
'
'   L' uso di questa routine deve essere preceduto da una
'   chiamata a DFT_1_Init per la preparazione delle tavole
'   dei seni e dei coseni.
'
'   I valori della Trasformata in R(), X() NON vengono
'   normalizzati sulla lunghezza del Segnale NVAL.
'
'   Gli NVAL dati nel vettore D() sono organizzati come D(0 To NVAL - 1).
'   Gli NFRE dati nei vettori R(), X(), S(), F() e Freq() sono organizzati
'   come Vettore(0 To NFRE - 1).
'
'   Vengono calcolati in R(0 To NFRE - 1), X(0 To NFRE - 1), S(0 To NFRE - 1)
'   e F(0 To NFRE - 1) solo i valori di frequenza contenuti in Freq() (vedi
'   routine di inizializzazione DFT_1_Init).
'
'   Algoritmi tratti da: "Digital Time Series Analysis" di R. Otnes
'   e L. Enochson - pg. 138.
'
    Dim JF&, JD&
'
    For JF = 0 To NFreqDFT_1
        r(JF) = 0#
        X(JF) = 0#
'
        For JD = 0 To NcDFT_1
            r(JF) = r(JF) + D(JD) * CosMat(JF, JD)
            X(JF) = X(JF) + D(JD) * SinMat(JF, JD)
        Next JD
    Next JF
'
    ' Calcolo dello Spettro di Ampiezza e delle Fasi:
    SMax = 0#
    ISMax = 0
    For JF = 0 To NFreqDFT_1
        S(JF) = Sqr(r(JF) * r(JF) + X(JF) * X(JF)) / NormDFT
        'F(JF) = DATAN2(-X(JF), R(JF))
        If (SMax < S(JF)) Then
            SMax = S(JF)
            ISMax = JF
        End If
    Next JF
'
    ' Calcolo del valor medio di S() con
    ' l' esclusione del valore corrispondente
    ' a ISMax:
    SMed = 0#
    For JF = 0 To NFreqDFT_1
        If (JF <> ISMax) Then SMed = SMed + S(JF)
    Next JF
    SMed = SMed / CDbl(NFreqDFT_1)
'
'
'
End Sub
Public Function SintesiFIR_WM(ByVal Tipo$, ByVal NFilt As Long, _
    ByVal F1 As Double, ByVal F2 As Double, ByVal WNome$, ByVal Par As Double) As Boolean
'
'   Sintesi del Filtro con Window Method.
'
'   Routine modificata per selezione toni.
'   --------------------------------------
'
'   Parametri in ingresso.
'    Tipo$:     Tipo di Filtro:
'                "Low Pass"
'                "High Pass"
'                "Band Pass"
'                "Band Stop"
'                "Differentiator"
'                "Hilbert Trans."
'    NFilt:     Numero di Coefficienti del Filtro.
'    F1:        Prima   Frequenza di Taglio (0 <= F1 < Fs/2).
'    F2:        Seconda Frequenza di Taglio (F1 < F2 <= Fs/2).
'    WNome$:    Nome del tipo di Window desiderato (vedi Function WinTipi()).
'    Par:       Parametro per certi tipi di Windows.
'
'   I coefficienti del filtro vengono calcolati
'   nel vettore Hc(0 To NFilt - 1).
'
    Dim I&, K&, KK&, NOdd&, NFilt1&, M$
    Dim Aa#, Ak#, Af1#, Af2#, Den#
    ReDim Hc(0 To NFilt - 1), SRE(-1 To NFilt - 1)
'
    On Error GoTo SintesiFIR_WM_ERR
'
    NOdd = NFilt Mod 2
    NFilt1 = Int(NFilt / 2)
    Aa = 0.5 * (1# - CDbl(NOdd))
'
    WF() = WinProf(WNome$, NFilt, Par)
'
'       Impulse response evaluation
'       ---------------------------
'
    If NOdd = 1 Then
        Select Case Tipo$
            Case "Low Pass"
            Hc(NFilt1) = 2# * F1
            Case "High Pass"
            Hc(NFilt1) = 1# - 2# * F1
            Case "Band Pass"
            Hc(NFilt1) = 2# * (F2 - F1)
            Case "Band Stop"
            Hc(NFilt1) = 1# - 2# * (F2 - F1)
            Case "Differentiator"
            Hc(NFilt1) = 0#
            Case "Hilbert Trans."
            Hc(NFilt1) = 0#
        End Select
    End If
'
    For I = 1 To NFilt1
        K = NFilt1 - I
        Ak = CDbl(I) - Aa
        Af1 = PI2 * Ak * F1
        Af2 = PI2 * Ak * F2
        Den = PI * Ak
        Select Case Tipo$
            Case "Low Pass"
            Hc(K) = Sin(Af1) / Den
            Case "High Pass"
            Hc(K) = (Sin(PI * Ak) - Sin(Af1)) / Den
            Case "Band Pass"
            Hc(K) = (Sin(Af2) - Sin(Af1)) / Den
            Case "Band Stop"
            Hc(K) = (Sin(PI * Ak) - Sin(Af2) + Sin(Af1)) / Den
            Case "Differentiator"
            Hc(K) = -(Sin(Af1) - Af1 * Cos(Af1)) / (Den * Den)
            Case "Hilbert Trans."
            Hc(K) = -(Cos(Af1) - Cos(Af2)) / Den
        End Select
'
        ' Windowing:
        Hc(K) = Hc(K) * WF(K)
    Next I
'
'
    For K = 0 To NFilt1 - 1
        KK = NFilt - 1 - K
        Hc(KK) = Switch(Tipo$ = "Differentiator", -Hc(K), _
                        Tipo$ = "Hilbert Trans.", -Hc(K), True, Hc(K))
    Next K
'
'
SintesiFIR_WM_ERR:
    SintesiFIR_WM = (Err.Number = 0)
    If (Err.Number <> 0) Then
        M$ = FiltroErrFIR$ & vbNewLine
        M$ = M$ & "Errore " & Str$(Err.Number) & vbNewLine
        M$ = M$ & Err.Description
        MsgBox M$, vbCritical, " modDSP: SintesiFIR_WM"
    End If
'
'
'
End Function
Public Function FIR(ByVal dIn As Single) As Single
'
'   Funzione di filtrazione FIR dei segnali.
'   Devono essere stati definiti, precedentemente, un vettore
'   Hc(0 To NCoeff - 1) contenente gli NCoeff coefficienti del
'   filtro ed un vettore SRE(-1 To NCoeff - 1) da usare come
'   registro a scorrimento dei campioni da filtrare.
'
'   Routine modificata per selezione toni.
'   --------------------------------------
'
    Dim K&, dOut!
'
    SRE(-1) = dIn
    For K = UBound(SRE) To 0 Step -1
        SRE(K) = SRE(K - 1)
        dOut = dOut + SRE(K) * Hc(K)
    Next K
'
    FIR = dOut
'
'
'
End Function
Public Sub DFT(ByVal NVAL As Long, D() As Double, ByRef r() As Double, _
    ByRef X() As Double, ByRef S() As Double, ByRef F() As Double, ByVal NFRE As Long)
'
'   Definizioni:
'
'   DFT = Discrete Fourier Transform.
'
'   Entra con:
'       NVAL   = Numero max. dei dati in D() da usare.
'       D()    = Vettore dei dati Reali del Segnale(t).
'       NFRE   = Numero  dei valori di Frequenza calcolati.
'
'   Esce  con:
'       R()    = Vettore dei valori Reali      della Trasformata(f).
'       X()    = Vettore dei valori Immaginari della Trasformata(f).
'       S()    = Vettore dello Spettro di Amp. della Trasformata(f).
'       F()    = Vettore delle Fasi            della Trasformata(f) [rad].
'
'   I valori della Trasformata in R(), X() NON vengono
'   normalizzati sulla lunghezza del Segnale NVAL.
'
'   Gli NVAL dati nel vettore D() sono organizzati come D(0 To NVAL - 1).
'   Gli NFRE dati nei vettori R(), X(), S() e F() sono organizzati come Vettore(0 To NFRE).
'
'   Viene calcolata in R(0 To NFRE), X(0 To NFRE) solo la prima
'   meta' dello spettro per i valori di frequenza  NFRE = NVAL/2
'   piu' la Componente Continua in R(0).
'
'   Algoritmi tratti da: "Digital Time Series Analysis" di R. Otnes
'   e L. Enochson - pg. 138.  Nessun tentativo e' stato fatto per
'   migliorare la velocita' di esecuzione (tipo look-up table e/o
'   valutazione ricorsiva delle funzioni trigonometriche).
'
    Dim JF&, JD&, PI2_V#, Omega#, OmT#, NFRE_D#
'
    'NFRE = Int(NVAL / 2)
    NFRE_D = CDbl(NVAL) / 2#
    PI2_V = PI2 / CDbl(NVAL)
'
    For JF = 0 To NFRE
        Omega = PI2_V * CDbl(JF)
        r(JF) = 0#
        X(JF) = 0#
'
        For JD = 0 To NVAL - 1
            OmT = Omega * CDbl(JD)
            r(JF) = r(JF) + D(JD) * Cos(OmT)
            X(JF) = X(JF) + D(JD) * Sin(OmT)
        Next JD
    Next JF
'
    ' Calcolo dello Spettro di Ampiezza e delle Fasi:
    S(0) = Abs(r(0)) / CDbl(NVAL)
    F(0) = IIf(r(0) < 0#, PI, 0#)
    For JF = 1 To NFRE - 1
        S(JF) = Sqr(r(JF) * r(JF) + X(JF) * X(JF)) / NFRE_D
        F(JF) = DATAN2(-X(JF), r(JF))
    Next JF
    If (NVAL Mod 2) = 0 Then
        S(NFRE) = Abs(r(NFRE)) / CDbl(NVAL)
        F(NFRE) = IIf(r(NFRE) < 0#, PI, 0#)
    Else
        S(NFRE) = Sqr(r(NFRE) * r(NFRE) + X(NFRE) * X(NFRE)) / NFRE_D
        F(NFRE) = DATAN2(-X(NFRE), r(NFRE))
    End If
'
'
'
End Sub
Public Sub FFT(ByVal NVAL As Long, ByRef r() As Double, ByRef X() As Double, _
    ByVal NFRE As Long, ByRef S() As Double, ByRef SMax As Double, ByRef ISMax As Long, _
    Optional ByRef SMed As Double, Optional ByRef SSig As Double)
'
'   Definizioni:
'
'   FFT = Fast Fourier Transform.
'
'   Routine modificata per selezione toni.
'   --------------------------------------
'
'   Entra con:
'       NVAL  = Numero max. di dati in R() da usare.
'       R()   = Vettore dei valori Reali del Segnale(t).
'       X()   = Vettore dei valori Immaginari del Segnale(t);
'               per segnali reali impostare a 0 tutti i valori
'               di questo vettore.
'               Viene sfruttata la proprieta' "transform-in-place"
'               di questa implementazione.
'       NFRE  = Numero dei valori di Frequenza da calcolare.
'
'   Esce  con:
'       R()   = Vettore dei valori Reali      della Trasformata(f).
'       X()   = Vettore dei valori Immaginari della Trasformata(f).
'       S()   = Vettore dello Spettro di Amp. della Trasformata(f).
'       SMax  = Valor massimo dello Spettro di Ampiezza.
'       ISMax = Indice del valor massimo dello Spettro di Ampiezza.
'       SMed  = Valor medio dei valori di S().
'       SSig  = Deviazione standard dei valori di S().
'
'   I valori della Trasformata in R(), X() NON vengono
'   normalizzati sulla lunghezza del Segnale 2^M.
'
'   Gli NVAL dati nei vettori R() e X() sono organizzati come Vettore(0 To NVAL - 1).
'   Gli NFRE dati nel vettore S() sono organizzati come S(0 To NFRE).
'
'   Tradotta in Basic da: "Theory and Application of Digital
'   Signal Processing" di L. Rabiner e B. Gold. - pg. 367.
'
    Dim M&, I&, NMN1&, J&, K&, L&, LE&, LE1&, IP&
    Dim Ur#, Ui#, Wr#, Wi#, Tr#, Ti#, Swap#
'
    M = CLng(Log(CDbl(NVAL)) / Log2)
    NMN1 = NVAL - 1
'
    J = 0
    For I = 0 To NMN1 - 1
        If (I < J) Then
            ' Swap R(J), R(I):
            Swap = r(J)
            r(J) = r(I)
            r(I) = Swap
            ' Swap X(J), X(I), solo per segnali complessi.
            ' ...
        End If
        K = NFRE
        Do While (K - 1 < J)
            J = J - K
            K = K / 2
        Loop
        J = J + K
    Next I
'
    For L = 1 To M
        LE = 2 ^ L
        LE1 = LE / 2
        Ur = 1#
        Ui = 0#
        Wr = Cos(PI / CDbl(LE1))
        Wi = Sin(PI / CDbl(LE1))
        For J = 0 To LE1 - 1
            For I = J To NVAL - 1 Step LE
                IP = I + LE1
                Tr = r(IP) * Ur - X(IP) * Ui
                Ti = r(IP) * Ui + X(IP) * Ur
                r(IP) = r(I) - Tr
                X(IP) = X(I) - Ti
                r(I) = r(I) + Tr
                X(I) = X(I) + Ti
            Next I
            Tr = Ur
            Ti = Ui
            Ur = Tr * Wr - Ti * Wi
            Ui = Tr * Wi + Ti * Wr
        Next J
    Next L
'
    ' Calcolo dello Spettro di Ampiezza
    ' e ricerca del valor massimo:
    Dim dSVal#, dSValQ#
    S(0) = Abs(r(0)) / CDbl(NVAL)
    SMax = S(0)
    ISMax = 0
    dSVal = S(0)
    dSValQ = S(0) * S(0)
    For J = 1 To NFRE - 1
        S(J) = Sqr(r(J) * r(J) + X(J) * X(J)) / CDbl(NFRE)
        If S(J) > SMax Then
            SMax = S(J)
            ISMax = J
        End If
        dSVal = dSVal + S(J)
        dSValQ = dSValQ + S(J) * S(J)
    Next J
    S(NFRE) = Abs(r(NFRE)) / CDbl(NVAL)
    dSVal = dSVal + S(NFRE)
    dSValQ = dSValQ + S(NFRE) * S(NFRE)
'
    ' Calcolo del valor medio e della
    ' deviazione standard di S():
    SMed = dSVal / CDbl(NFRE + 1)
    SSig = CDbl(NFRE + 1) * dSValQ - dSVal * dSVal
    SSig = Sqr(SSig / (CDbl(NFRE + 1) * (CDbl(NFRE))))
'
'
'
End Sub
Public Sub FFT_2(ByVal NVAL As Long, ByRef r() As Double, ByRef X() As Double, _
    ByVal NFRE As Long, ByVal IF1 As Long, ByVal IF2 As Long, ByRef S() As Double, _
    ByRef SMax As Double, ByRef ISMax As Long, _
    Optional ByRef SMed As Double, Optional ByRef SSig As Double)
'
'   Definizioni:
'
'   FFT = Fast Fourier Transform.
'
'   Routine modificata per selezione toni.
'   --------------------------------------
'
'   Entra con:
'       NVAL  = Numero max. di dati in R() da usare.
'       R()   = Vettore dei valori Reali del Segnale(t).
'       X()   = Vettore dei valori Immaginari del Segnale(t);
'               per segnali reali impostare a 0 tutti i valori
'               di questo vettore.
'               Viene sfruttata la proprieta' "transform-in-place"
'               di questa implementazione.
'       NFRE  = Numero massimo di valori di Frequenza da calcolare.
'       IF1   = Indice del primo valore di frequenza da calcolare: 0 < IF1 < IF2.
'       IF2   = Indice dell' ultimo valore di frequenza da calcolare: IF1 < IF2 < NFRE.
'
'   Esce  con:
'       R()   = Vettore dei valori Reali      della Trasformata(f).
'       X()   = Vettore dei valori Immaginari della Trasformata(f).
'       S()   = Vettore dello Spettro di Amp. della Trasformata(f).
'       SMax  = Valor massimo dello Spettro di Ampiezza.
'       ISMax = Indice del valor massimo dello Spettro di Ampiezza.
'       SMed  = Valor medio di S() considerato come una distribuzione di probabilita'.
'       SSig  = Deviazione standard di S() considerato come una distribuzione di probabilita'.
'
'   I valori della Trasformata in R(), X() NON vengono
'   normalizzati sulla lunghezza del Segnale 2^M.
'
'   Gli NVAL dati nei vettori R() e X() sono organizzati come Vettore(0 To NVAL - 1).
'   Gli NFRE dati nel vettore S() sono organizzati come S(0 To NFRE)
'   e sono validi solo i valori S(J) con IF1 <= J <= IF2.
'
'   Tradotta in Basic da: "Theory and Application of Digital
'   Signal Processing" di L. Rabiner e B. Gold. - pg. 367.
'
    Dim M&, I&, NMN1&, J&, K&, L&, LE&, LE1&, IP&
    Dim Ur#, Ui#, Wr#, Wi#, Tr#, Ti#, Swap#
'
    M = CLng(Log(CDbl(NVAL)) / Log2)
    NMN1 = NVAL - 1
'
    J = 0
    For I = 0 To NMN1 - 1
        If (I < J) Then
            ' Swap R(J), R(I):
            Swap = r(J)
            r(J) = r(I)
            r(I) = Swap
            ' Swap X(J), X(I), solo per segnali complessi.
            ' ...
        End If
        K = NFRE
        Do While (K - 1 < J)
            J = J - K
            K = K / 2
        Loop
        J = J + K
    Next I
'
    For L = 1 To M
        LE = 2 ^ L
        LE1 = LE / 2
        Ur = 1#
        Ui = 0#
        Wr = Cos(PI / CDbl(LE1))
        Wi = Sin(PI / CDbl(LE1))
        For J = 0 To LE1 - 1
            For I = J To NVAL - 1 Step LE
                IP = I + LE1
                Tr = r(IP) * Ur - X(IP) * Ui
                Ti = r(IP) * Ui + X(IP) * Ur
                r(IP) = r(I) - Tr
                X(IP) = X(I) - Ti
                r(I) = r(I) + Tr
                X(I) = X(I) + Ti
            Next I
            Tr = Ur
            Ti = Ui
            Ur = Tr * Wr - Ti * Wi
            Ui = Tr * Wi + Ti * Wr
        Next J
    Next L
'
    ' Calcolo dello Spettro di Ampiezza
    ' e ricerca del valor massimo:
    'IF1 = 1: IF2 = NFRE
    Dim dSVal#, dSVI#, dSValQ#
    S(IF1) = Sqr(r(IF1) * r(IF1) + X(IF1) * X(IF1)) / CDbl(NFRE)
    SMax = S(IF1)
    ISMax = IF1
    dSVal = S(IF1)
    dSVI = CDbl(IF1) * S(IF1)
    For J = IF1 + 1 To IF2
        S(J) = Sqr(r(J) * r(J) + X(J) * X(J)) / CDbl(NFRE)
        If S(J) > SMax Then
            SMax = S(J)
            ISMax = J
        End If
        dSVal = dSVal + S(J)
        dSVI = dSVI + CDbl(J) * S(J)
    Next J
'
    ' Calcolo del valor medio e della deviazione
    ' standard della distribuzione di probabilita' S():
    If dSVal > 0 Then
        SMed = dSVI / dSVal
        For J = IF1 To IF2
            SSig = SSig + ((J - SMed) ^ 2) * S(J)
        Next J
        SSig = Sqr(SSig / dSVal)
    Else
        SMed = -1#
        SSig = -1#
    End If
'
'
'
End Sub
Public Sub FFT_1(ByVal NVAL As Long, ByRef r() As Double, ByRef X() As Double, _
    ByVal NFRE As Long, ByVal IF1 As Long, ByVal IF2 As Long, ByRef S() As Double, _
    ByRef SMax As Double, ByRef ISMax As Long, _
    Optional ByRef SMed As Double, Optional ByRef SSig As Double)
'
'   Definizioni:
'
'   FFT = Fast Fourier Transform.
'
'   Routine modificata per selezione toni.
'   --------------------------------------
'
'   Entra con:
'       NVAL  = Numero max. di dati in R() da usare.
'       R()   = Vettore dei valori Reali del Segnale(t).
'       X()   = Vettore dei valori Immaginari del Segnale(t);
'               per segnali reali impostare a 0 tutti i valori
'               di questo vettore.
'               Viene sfruttata la proprieta' "transform-in-place"
'               di questa implementazione.
'       NFRE  = Numero massimo di valori di Frequenza da calcolare.
'       IF1   = Indice del primo valore di frequenza da calcolare: 0 < IF1 < IF2.
'       IF2   = Indice dell' ultimo valore di frequenza da calcolare: IF1 < IF2 < NFRE.
'
'   Esce  con:
'       R()   = Vettore dei valori Reali      della Trasformata(f).
'       X()   = Vettore dei valori Immaginari della Trasformata(f).
'       S()   = Vettore dello Spettro di Amp. della Trasformata(f).
'       SMax  = Valor massimo dello Spettro di Ampiezza.
'       ISMax = Indice del valor massimo dello Spettro di Ampiezza.
'       SMed  = Valor medio dei valori di S() con esclusione del valore ISMax.
'       SSig  = Deviazione standard dei valori di S().
'
'   I valori della Trasformata in R(), X() NON vengono
'   normalizzati sulla lunghezza del Segnale 2^M.
'
'   Gli NVAL dati nei vettori R() e X() sono organizzati come Vettore(0 To NVAL - 1).
'   Gli NFRE dati nel vettore S() sono organizzati come S(0 To NFRE)
'   e sono validi solo i valori S(J) con IF1 <= J <= IF2.
'
'   Tradotta in Basic da: "Theory and Application of Digital
'   Signal Processing" di L. Rabiner e B. Gold. - pg. 367.
'
    Dim M&, I&, NMN1&, J&, K&, L&, LE&, LE1&, IP&
    Dim Ur#, Ui#, Wr#, Wi#, Tr#, Ti#, Swap#
'
    M = CLng(Log(CDbl(NVAL)) / Log2)
    NMN1 = NVAL - 1
'
    J = 0
    For I = 0 To NMN1 - 1
        If (I < J) Then
            ' Swap R(J), R(I):
            Swap = r(J)
            r(J) = r(I)
            r(I) = Swap
            ' Swap X(J), X(I), solo per segnali complessi.
            ' ...
        End If
        K = NFRE
        Do While (K - 1 < J)
            J = J - K
            K = K / 2
        Loop
        J = J + K
    Next I
'
    For L = 1 To M
        LE = 2 ^ L
        LE1 = LE / 2
        Ur = 1#
        Ui = 0#
        Wr = Cos(PI / CDbl(LE1))
        Wi = Sin(PI / CDbl(LE1))
        For J = 0 To LE1 - 1
            For I = J To NVAL - 1 Step LE
                IP = I + LE1
                Tr = r(IP) * Ur - X(IP) * Ui
                Ti = r(IP) * Ui + X(IP) * Ur
                r(IP) = r(I) - Tr
                X(IP) = X(I) - Ti
                r(I) = r(I) + Tr
                X(I) = X(I) + Ti
            Next I
            Tr = Ur
            Ti = Ui
            Ur = Tr * Wr - Ti * Wi
            Ui = Tr * Wi + Ti * Wr
        Next J
    Next L
'
    ' Calcolo dello Spettro di Ampiezza
    ' e ricerca del valor massimo:
    Dim dSVal#, dSValQ#
    S(IF1) = Sqr(r(IF1) * r(IF1) + X(IF1) * X(IF1)) / CDbl(NFRE)
    SMax = S(IF1)
    ISMax = IF1
    dSVal = S(IF1)
    dSValQ = S(IF1) * S(IF1)
    For J = IF1 + 1 To IF2
        S(J) = Sqr(r(J) * r(J) + X(J) * X(J)) / CDbl(NFRE)
        If S(J) > SMax Then
            SMax = S(J)
            ISMax = J
        End If
        dSVal = dSVal + S(J)
        dSValQ = dSValQ + S(J) * S(J)
    Next J
'
    ' Calcolo del valor medio di S() con
    ' l' esclusione del valore corrispondente
    ' a ISMax:
    SMed = 0#
    For J = IF1 To IF2
        If J <> ISMax Then SMed = SMed + S(J)
    Next J
    SMed = SMed / CDbl(IF2 - IF1)
'
    ' Calcolo della deviazione standard di S():
    SSig = CDbl(IF2 - IF1 + 2) * dSValQ - dSVal * dSVal
    SSig = Sqr(SSig / (CDbl(IF2 - IF1 + 2) * (CDbl(IF2 - IF1 + 1))))
'
'
'
End Sub
Public Function WinProf(ByVal WNome$, ByVal NWind As Long, _
    Optional ByVal Par As Double) As Double()
'
'   Calcola e ritorna un vettore(0 To NWind - 1) con i
'   coefficienti della Window richiesta.
'    WNome$:    nome della Window richiesta.
'    NWind:     N di coefficienti richiesto.
'    Par:       parametro richiesto per certi tipi di Window.
'
    Dim K&, KK&, NOdd&, NWind1&
    Dim Bb#, Aa#, Ak#, Arg#, ArgL#, ArgW#
    ReDim WF_I#(0 To NWind - 1)
'
    NOdd = NWind Mod 2
    NWind1 = Int(NWind / 2)
    If NOdd = 1 Then WF_I(NWind1) = 1#
'
    Aa = 0.5 * (1# - CDbl(NOdd))
    Bb = CDbl(NWind1) - Aa
'
    ' Calcola la prima meta' del vettore:
    Select Case WNome$
        Case "Blackman"
        For K = 0 To NWind1 - 1
            WF_I(K) = 0.42 - 0.5 * Cos(PI2 * CDbl(K) / CDbl(NWind - 1)) _
                    + 0.08 * Cos(2# * PI2 * CDbl(K) / CDbl(NWind - 1))
        Next K
'
        Case "Hamming generalizzata"
        For K = 0 To NWind1 - 1
            WF_I(K) = Par - (1# - Par) * Cos(PI2 * CDbl(K) / CDbl(NWind - 1))
        Next K
'
        Case "Hamming"
        ' E' l' "Hamming generalizzata" con Par = 0.54:
        For K = 0 To NWind1 - 1
            WF_I(K) = 0.54 - 0.46 * Cos(PI2 * CDbl(K) / CDbl(NWind - 1))
        Next K
'
        Case "Hanning"
        ' E' l' "Hamming generalizzata" con Par = 0.5:
        For K = 0 To NWind1 - 1
            WF_I(K) = 0.5 * (1# - Cos(PI2 * CDbl(K) / CDbl(NWind - 1)))
        Next K
'
        Case "Kaiser"
        For K = 0 To NWind1 - 1
            Ak = Par * Sqr(1# - ((2# * CDbl(K) / CDbl(NWind - 1)) - 1#) ^ 2)
            WF_I(K) = I0_Kaiser(Ak) / I0_Kaiser(Par)
        Next K
'
        Case "Lanczos"
        ArgL = PI / Bb
        For K = 0 To NWind1 - 1
            Ak = CDbl(NWind1 - K) - Aa
            Arg = ArgL * Ak
            WF_I(K) = Abs(Sin(Arg) / Arg) ^ Par
        Next K
'
        Case "Rettangolare"
        For K = 0 To NWind1 - 1
            WF_I(K) = 1#
        Next K
'
        Case "Triangolare"
        For K = 0 To NWind1 - 1
            WF_I(K) = CDbl(K) / CDbl(NWind1)
        Next K
'
        Case "Weber"
        ArgW = 1.5 * Par / Bb
        For K = 0 To NWind1 - 1
            Ak = CDbl(NWind1 - K) - Aa
            Arg = ArgW * Ak
            If (Arg <= 0.75) Then
                WF_I(K) = A0 + A1 * Arg + A2 * Arg ^ 2 + A3 * Arg ^ 3
            Else
                WF_I(K) = B0 + B1 * Arg + B2 * Arg ^ 2 + B3 * Arg ^ 3
            End If
        Next K
    End Select
'
'
    ' Completa il vettore
    ' con la seconda meta':
    For K = 0 To NWind1 - 1
        KK = NWind - 1 - K
        WF_I(KK) = WF_I(K)
    Next K
'
    WinProf = WF_I()
'
'
'
End Function
Private Function I0_Kaiser(ByVal X As Double, Optional ByRef N As Long) As Double
'
'   Calcola e ritorna la funzione di Bessel modificata del primo
'   tipo e di ordine zero per 0 <= X <= 20.  N e' il numero
'   di iterazioni impiegate per la convergenza.
'    Da: Theory and Application of Digital Signal Processing.
'        L. R. Rabiner, B. Gold   -   pg. 103
'
    Dim Y#, E#, de#, SDE#
    Const t# = 0.00000001
'
    Y = X / 2#
    E = 1#
    de = 1#
    For N = 1 To 25
        de = de * Y / CDbl(N)
        SDE = de ^ 2
        E = E + SDE
        If E * t - SDE > 0# Then
            I0_Kaiser = E
            Exit Function
        End If
    Next N
'
    N = 0
'
'
'
End Function
Public Function WinTipi() As Window_Type()
'
'   Imposta i parametri dei tipi di
'   "Window" disponibili:
'    Profilo da File:       WinTipi(-1)
'    Blackman:              WinTipi(0)
'    Hamming generalizzata: WinTipi(1)
'    Hamming:               WinTipi(2)
'    Hanning:               WinTipi(3)
'    Kaiser:                WinTipi(4)
'    Lanczos:               WinTipi(5)
'    Rettangolare:          WinTipi(6)
'    Triangolare:           WinTipi(7)
'    Weber:                 WinTipi(8)
'
    Dim WTipi(-1 To 8) As Window_Type
'
    WTipi(-1).Nome = "--> Nome del File"
    WTipi(-1).PMin = -1
    WTipi(-1).PMax = -1
'
    WTipi(0).Nome = "Blackman"
    WTipi(0).PMin = -1
    WTipi(0).PMax = -1
'
    WTipi(1).Nome = "Hamming generalizzata"
    WTipi(1).PMin = 0.5
    WTipi(1).PMax = 1#
    WTipi(1).PCor = 0.5
'
    WTipi(2).Nome = "Hamming"
    WTipi(2).PMin = -1
    WTipi(2).PMax = -1
'
    WTipi(3).Nome = "Hanning"
    WTipi(3).PMin = -1
    WTipi(3).PMax = -1
'
    WTipi(4).Nome = "Kaiser"
    WTipi(4).PMin = 0.5
    WTipi(4).PMax = 20#
    WTipi(4).PCor = 7.865
'
    WTipi(5).Nome = "Lanczos"
    WTipi(5).PMin = 0.5
    WTipi(5).PMax = 3.5
    WTipi(5).PCor = 1#
'
    WTipi(6).Nome = "Rettangolare"
    WTipi(6).PMin = -1
    WTipi(6).PMax = -1
'
    WTipi(7).Nome = "Triangolare"
    WTipi(7).PMin = -1
    WTipi(7).PMax = -1
'
    WTipi(8).Nome = "Weber"
    WTipi(8).PMin = 0.5
    WTipi(8).PMax = 1#
    WTipi(8).PCor = 1#
'
    WinTipi = WTipi()
'
'
'
End Function
Public Sub DFT_1_Init(ByVal NVAL As Long, ByVal NFRE As Long, _
    Freq() As Double, ByVal Fs As Double)
'
'   Routine di inizializzazione (preparazione delle
'   tavole dei seni e coseni) per la DFT_1:
'
'   Entra con:
'       NVAL   = Numero di dati in D().
'       NFRE   = Numero dei valori di Frequenza da calcolare.
'       Freq() = Vettore con le frequenze alle quali calcolare la trasformata [Hz].
'       Fs     = Frequenza di campionamento del segnale D(t) [Hz].
'
'   I valori di frequenza contenuti in Freq() devono essere:
'    0 < Freq(JF) < Fs/2  con 0 <= JF <= NFRE - 1.
'
    Dim JF&, JD&, Omega#, OmT#
    ReDim SinMat(0 To NFRE - 1, 0 To NVAL - 1)
    ReDim CosMat(0 To NFRE - 1, 0 To NVAL - 1)
'
    NcDFT_1 = NVAL - 1          ' N di campioni - 1 per il calcolo della DFT.
    NFreqDFT_1 = NFRE - 1       ' N di frequenze - 1 a cui calcolare la DFT.
    NormDFT = CDbl(NVAL) / 2#   ' Fattore di normalizzazione sul N di campioni.
'
    For JF = 0 To NFRE - 1
        Omega = PI2 * Freq(JF) / Fs
'
        For JD = 0 To NVAL - 1
            OmT = Omega * CDbl(JD)
            CosMat(JF, JD) = Cos(OmT)
            SinMat(JF, JD) = Sin(OmT)
        Next JD
    Next JF
'
'
'
End Sub
Public Function SPM(ByVal Xn As Single) As Single
'
'   Funzione per il calcolo con convoluzione diretta dello
'   spettro mobile (sliding spectrum) su NcSpm campioni.
'   Devono essere stati definiti, precedentemente, il vettore
'   dei coefficienti Z1() ed un vettore SmRE(0 To NcSpm - 1) da
'   usare come registro a scorrimento dei campioni del segnale.
'
    Dim K&
'
    For K = NcSpm - 1 To 1 Step -1
        SmRE(K) = SmRE(K - 1)
    Next K
    SmRE(0) = CDbl(Xn)
'
    Sn = CCmp(SmRE(0), 0#)
    For K = 1 To NcSpm - 1
        Sn = CSom(Sn, CMol(CCmp(SmRE(K), 0#), z1(K)))
    Next K
'
    SPM = CSng(CAbs(Sn)) / CDbl(NcSpm)
'
'
'
End Function
Public Sub SPM_MF_R(ByVal IMed As Long, ByVal Xn As Single, _
    ByRef SpmMF() As Double, ByRef MedMF As Double)
'
'   Routine per il calcolo ricorsivo dello spettro mobile (sliding
'   spectrum) multi frequenza su NcSpm campioni.
'   Devono essere stati definiti, precedentemente, i coefficienti
'   z1_MF_1(), z1_MF_N() ed un vettore SmRE(0 To NcSpm) da usare come
'   registro a scorrimento dei campioni del segnale.
'   Ritorna anche il valore della media (o del valor efficace) mobile
'   degli ultimi NcSpm campioni.
'   In questa routine, per ragioni di velocita', le funzioni di numeri
'   complessi sono state sostituite con il loro sviluppo in linea.
'
    Dim K&
'
    ' SmRE() usato come buffer circolare:
    Ic = ((Ic + 1) Mod NcSpm1)
    Kc = ((Ic + 1) Mod NcSpm1)
    SmRE(Ic) = CDbl(Xn)
'
    For K = 0 To NFreqSPM_1
        'XZ1_N = CMol(CCmp(SmRE(Kc), 0#), z1_MF_N)
        Xz1_N.Re = SmRE(Kc) * z1_MF_N(K).Re
        Xz1_N.Im = SmRE(Kc) * z1_MF_N(K).Im
        'SnI = CDif(CCmp(SmRE(Ic), 0#), XZ1_N)
        SnI.Re = SmRE(Ic) - Xz1_N.Re
        SnI.Im = -Xz1_N.Im
'
        'YZ1_1 = CMol(Sn_MF_1(K), z1_MF_1(K))
        Yz1_1.Re = Sn_MF_1(K).Re * z1_MF_1(K).Re - Sn_MF_1(K).Im * z1_MF_1(K).Im
        Yz1_1.Im = Sn_MF_1(K).Re * z1_MF_1(K).Im + Sn_MF_1(K).Im * z1_MF_1(K).Re
        'Sn = CSom(SnI, YZ1_1)
        Sn.Re = SnI.Re + Yz1_1.Re
        Sn.Im = SnI.Im + Yz1_1.Im
        Sn_MF_1(K) = Sn
'
        'SpmMF(K) = CAbs(Sn) / NormSpm
        SpmMF(K) = Sqr(Sn.Re * Sn.Re + Sn.Im * Sn.Im) / NormSpm
    Next K
'
    ' Calcolo la media/valor efficace mobile:
    If (NcSpm < IMed) Then
        ' Media mobile del valore assoluto del segnale:
        'VMed = VMed + Abs(SmRE(Ic)) - Abs(SmRE(Kc))
        'MedMF = VMed / CDbl(NcSpm)
'
        ' Valore efficace mobile:
        SqRE(Ic) = SmRE(Ic) * SmRE(Ic)
        VMed = VMed + SqRE(Ic) - SqRE(Kc)
        If (VMed < 0) Then VMed = 0     ' VMed NON puo' essere negativo: se lo e'
                                        ' e' a causa degli errori di troncamento
                                        ' e quindi lo riazzero.
        MedMF = Sqr(VMed / CDbl(NcSpm))
'
    Else
        ' Gestisco l' inizio del segnale:
        If (IMed = 1) Then VMed = 0#
'
        ' Media mobile del valore assoluto del segnale:
        'VMed = VMed + Abs(SmRE(Ic))
        'MedMF = VMed / CDbl(IMed)
'
        ' Valore efficace mobile:
        SqRE(Ic) = SmRE(Ic) * SmRE(Ic)
        VMed = VMed + SqRE(Ic)
        MedMF = Sqr(VMed / CDbl(IMed))
    End If
'
'
'
End Sub
Public Function SPM_R(ByVal Xn As Single) As Single
'
'   Funzione per il calcolo ricorsivo dello spettro mobile
'   (sliding spectrum) su NcSpm campioni.
'   Devono essere stati definiti, precedentemente, i coefficienti
'   Z1_1, Z1_N ed un vettore SmRE(0 To NcSpm) da usare come
'   registro a scorrimento dei campioni del segnale.
'
    Dim K&
'
    ' SmRE() usato come buffer circolare:
    Ic = ((Ic + 1) Mod NcSpm1)
    Kc = ((Ic + 1) Mod NcSpm1)
    SmRE(Ic) = CDbl(Xn)
'
    ' XZ1_N = CMol(CCmp(SmRE(NcSpm), 0#), Z1_N):
    Xz1_N.Re = SmRE(Kc) * z1_N.Re
    Xz1_N.Im = SmRE(Kc) * z1_N.Im
'
    ' SnI = CDif(CCmp(SmRE(0), 0#), XZ1_N):
    SnI.Re = SmRE(Ic) - Xz1_N.Re
    SnI.Im = -Xz1_N.Im
'
    'YZ1_1 = CMol(Sn_MF_1(K), z1_MF_1(K))
    Yz1_1 = CMol(Sn_1, z1_1)
    Sn = CSom(SnI, Yz1_1)
    Sn_1 = Sn
'
    SPM_R = CAbs(Sn) / NormSpm
'
'
'
End Function
Public Sub SPM_MF_Init(ByVal NcSpm_I As Long, ByVal NFRE As Long, _
    Freq() As Double, ByVal Fs As Double)
'
'   Calcolo dei coefficienti necessari alla implementazione ricorsiva,
'   multi frequenza dello spettro mobile su NcSpm campioni.
'   Le frequenze, a cui calcolare gli spettri, devono essere definite nel
'   vettore Freq(0 to NFreqSPM_1); Fs e' la frequenza di campionamento:
'
    Dim I&, OmegaT#
'
    NcSpm = NcSpm_I             ' N di campioni per il calcolo dello spettro mobile.
    NcSpm1 = NcSpm + 1
    NFreqSPM_1 = NFRE - 1       ' N di frequenze - 1 a cui calcolare gli spettri.
    ReDim z1(0 To 0)            ' e^j*omega*T.
    ReDim SmRE(0 To NcSpm)      ' Registro a scorrimento dei campioni del segnale.
    ReDim SqRE(0 To NcSpm)      ' Registro a scorrimento dei campioni del segnale al quadrato.
    NormSpm = CDbl(NcSpm) / 2#  ' Fattore di normalizzazione sul N di campioni.
    VMed = 0#                   ' Valore medio corrente.
'
    ReDim Sn_MF_1(0 To NFreqSPM_1)
    ReDim z1_MF_1(0 To NFreqSPM_1)
    ReDim z1_MF_N(0 To NFreqSPM_1)
'
    For I = 0 To NFreqSPM_1
        OmegaT = PI2 * Freq(I) / Fs     ' Per frequenze in [Hz].
        'OmegaT = PI2 * Freq(I)          ' Per frequenze in [f/fs].
        z1(0) = CExp(CCmp(0#, OmegaT))
'
        ' Per calcolo ricorsivo:
        Sn_MF_1(I) = CCmp(0#, 0#)
        z1_MF_1(I) = CPtN(z1(0), -1)
        z1_MF_N(I) = CPtN(z1(0), -NcSpm)
    Next I
'
'
'
End Sub
Public Sub SPM_Init(ByVal NcSpm_I As Long, ByVal Freq As Double, ByVal Fs As Double)
'
'   Calcolo dei coefficienti necessari alle due implementazioni dello spettro mobile.
'   Freq e' la frequenza alla quale calcolare lo spettro mobile; Fs e' la frequenza
'   di campionamento:
'
    Dim I&, OmegaT#
'
    NcSpm = NcSpm_I             ' N di campioni per il calcolo dello spettro mobile.
    NcSpm1 = NcSpm + 1
    ReDim z1(0 To NcSpm)        ' Vettore dei coefficienti di calcolo con
                                ' convoluzione diretta dello spettro mobile;
                                ' z1(0) = e^j*omega*T.
    ReDim SmRE#(0 To NcSpm)     ' Registro a scorrimento dei campioni del segnale.
    NormSpm = CDbl(NcSpm) / 2#  ' Fattore di normalizzazione sul N di campioni.
'
    OmegaT = PI2 * Freq / Fs    ' Per frequenza in [Hz].
    'OmegaT = PI2 * Freq         ' Per frequenza in [f/fs].
    z1(0) = CExp(CCmp(0#, OmegaT))
'
    ' Per calcolo con convoluzione diretta:
    For I = 1 To NcSpm - 1
        z1(I) = CPtN(z1(0), -I)
    Next I
'
    ' Per calcolo ricorsivo:
    Sn_1 = CCmp(0#, 0#)
    z1_1 = CPtN(z1(0), -1)
    z1_N = CPtN(z1(0), -NcSpm)
'
'
'
End Sub
